/*
	illustration de quelques fonctions openGL

	mailto:jciehl@bat710.univ-lyon1.fr
*/

#include <unistd.h>
#include <GL/glut.h>
#include <math.h>
#include <time.h>

// variables globales ... glut oblige !

// position et orientation du point de vue
float camera_x, camera_y, camera_z;
float camera_angle;

// timer pour l'animation
int timer;
int timer_start;
int timer_rate;

// conversion degres vers radians
float radians(float d)
{
	return d * 0.0174532925;
}

// affiche un cube dans le repere courant
void cube(float x, float y, float z, float edge)
{
	float e2= edge / 2.;

	// le cube est affiche comme 4 quadrangles disjoints
	glBegin(GL_QUADS);
	
	// quandrangle
	glVertex3f(x-e2, y+e2, z-e2);
	glVertex3f(x+e2, y+e2, z-e2);
	glVertex3f(x+e2, y-e2, z-e2);
	glVertex3f(x-e2, y-e2, z-e2);

	// quandrangle
	glVertex3f(x-e2, y-e2, z+e2);
	glVertex3f(x+e2, y-e2, z+e2);
	glVertex3f(x+e2, y+e2, z+e2);
	glVertex3f(x-e2, y+e2, z+e2);

	// quandrangle
	glVertex3f(x-e2, y+e2, z-e2);
	glVertex3f(x-e2, y+e2, z+e2);
	glVertex3f(x-e2, y-e2, z+e2);
	glVertex3f(x-e2, y-e2, z-e2);

	// quandrangle
	glVertex3f(x+e2, y+e2, z-e2);
	glVertex3f(x+e2, y+e2, z+e2);
	glVertex3f(x+e2, y-e2, z+e2);
	glVertex3f(x+e2, y-e2, z-e2);
	glEnd();	// ferme la description de la primitive
}

// affiche la scene complete
void scene(void)
{
	// utilise le temps courant pour modifier une caracteristique de chaque objet
	
	glPushMatrix();
	glRotatef((timer*2)%180, 0., 1., 0);
	glColor3f ((float) (timer % 300) / 300., 1.0, (float) (timer % 100) / 100.);
	cube(0, 10*cos(radians(timer % 360)), 0, 2);
	glPopMatrix();

	glColor3f (1., 0., 0.);
	cube(10, 0, 0, 2+sin(radians(timer % 360)));

	glColor3f(0., 0., 1.);
	cube(0, 0, 10, 2+cos(radians(timer % 180)));
}

// callback de glut : affichage
void display(void)
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// efface l'ecran et le z-buffer

	glMatrixMode(GL_MODELVIEW);		// selectionne la matrice de la scene
	glLoadIdentity();				// reinitialise les transformations

	// positionne la scene par rapport a la camera qui est restee en 0,0,0
	glRotatef(-camera_angle, 0., 1., 0);
	glTranslatef(-camera_x, -camera_y, -camera_z);

	// desssine la scene
	scene();

	// termine l'affichage
    glFlush();

	// echange le buffer de dessin et le buffer d'affichage
    glutSwapBuffers();
}

// callback de glut : appellee regulierement
void anime(void)
{
	int c;

#if 1	
	usleep(1000000. / timer_rate);
	timer++;
	// demande un nouvel affichage si le temps ecoule correspond a la frequence d'animation definie
	glutPostRedisplay();
#else
	
	c= clock();
	if((c - timer_start)*timer_rate > CLOCKS_PER_SEC)
	{
		timer++;
		timer_start= c;
		
		// demande un nouvel affichage si le temps ecoule correspond a la frequence d'animation definie
		glutPostRedisplay();
	}
#endif
}

// callback de glut : gestion du clavier (touches speciales)
void keyboard(int key, int x, int y)
{
	switch(key)
	{
		case GLUT_KEY_UP:
			// deplacement de la camera dans la direction de la visee
			camera_z-= cos(radians(camera_angle));
			camera_x-= sin(radians(camera_angle));
		break;

		case GLUT_KEY_DOWN:
			// deplacement de la camera dans la direction opposee de la visee
			camera_z+= cos(radians(camera_angle));
			camera_x+= sin(radians(camera_angle));
		break;

		case GLUT_KEY_LEFT:
			// rotation de la camera sur Y-
			camera_angle+= 4.;
			if(camera_angle >= 360.)
				camera_angle= 0.;
		break;

		case GLUT_KEY_RIGHT:
			// rotation de la camera sur Y+
			camera_angle-= 4.;
			if(camera_angle < 0.)
				camera_angle= 359.;
		break;

		case GLUT_KEY_PAGE_UP:
			// accelere l'animation
			timer_rate+= 5;
			if(timer_rate > 100)
				timer_rate= 100;
		break;
		
		case GLUT_KEY_PAGE_DOWN:
			// ralentit l'animation
			timer_rate-= 5;
			if(timer_rate < 5)
				timer_rate= 5;
		break;
		
		default:
		break;
	}

	// provoque un affichage (appel du callback display)
	glutPostRedisplay();
}

// mise en place des parametres d'openGL
void init (void)
{
	// precise la couleur pour effacer l'ecran
    glClearColor (0.0, 0.0, 0.0, 0.0);
	
	// precise la profondeur pour effacer le z-buffer
	glClearDepth(1.0);
	
	// demande a effectuer un test sur le z-buffer pendant l'affichage
	glEnable(GL_DEPTH_TEST);
	// precise le test 
	glDepthFunc(GL_LEQUAL);
	
	// definit la projection
    glMatrixMode(GL_PROJECTION);	// selectionne la matrice de visualisation
    glLoadIdentity();				// reintialise les transformations

#if 0
    glOrtho(-1000.0, 1000.0, -1000.0, 1000.0, -1000.0, 1000.0);	// definit une projection orthographique
#else
	gluPerspective(50., 1., 1., 100.);
#endif

	// position de la camera au debut de l'animation
	camera_x= 0.;
	camera_y= 0.;
	camera_z= 50.;
	camera_angle= 0.;

	// gestion de la vitesse de l'animation
	timer= 0;
	timer_start= clock();
	timer_rate= 50;
}


int main(int argc, char** argv)
{
    glutInit(&argc, argv);
	
	// initialisation du mode d'affichage :
	//   GLUT_DOUBLE demande 2 buffers, un pour dessiner sans afficher, l'autre pour afficher
	//   GLUT_DEPTH demande 1 z-buffer
	//   GLUT_RGB demande un RGB 24 bits
    glutInitDisplayMode (GLUT_DOUBLE | GLUT_DEPTH | GLUT_RGB);
    
	glutInitWindowSize (400, 400);
    glutInitWindowPosition (100, 100);
    glutCreateWindow ("Ca bouge !!! (fleches, page up down)");
    init();
    // enregistre l'action des evenements claviers
	glutSpecialFunc(keyboard);
	
	// enregistre l'action des evenements d'affichage
    glutDisplayFunc(display);
	
	// enregistre l'action a executer en manque d'evenements
    glutIdleFunc(anime);
	
	// lance la boucle de gestion des evenements
    glutMainLoop();
    return 0;
}

