/*
	minigl.c
	squelette openGL/GLUT

	openGL programming guide

	compilez avec :
	gcc -o minigl minigl.c -lGL -lglut

*/


#include <GL/glut.h>
#include <math.h>
#include <time.h>

float camera_x, camera_y, camera_z;
float camera_angle;

int timer;
int timer_start;
int timer_rate;

float radians(float d)
{
	return d * 0.0174532925;
}

void cube(float x, float y, float z, float edge)
{
	float e2= edge / 2.;

	glBegin(GL_QUADS);
	glVertex3f(x-e2, y+e2, z-e2);
	glVertex3f(x+e2, y+e2, z-e2);
	glVertex3f(x+e2, y-e2, z-e2);
	glVertex3f(x-e2, y-e2, z-e2);

	glVertex3f(x-e2, y-e2, z+e2);
	glVertex3f(x+e2, y-e2, z+e2);
	glVertex3f(x+e2, y+e2, z+e2);
	glVertex3f(x-e2, y+e2, z+e2);

	glVertex3f(x-e2, y+e2, z-e2);
	glVertex3f(x-e2, y+e2, z+e2);
	glVertex3f(x-e2, y-e2, z+e2);
	glVertex3f(x-e2, y-e2, z-e2);

	glVertex3f(x+e2, y+e2, z-e2);
	glVertex3f(x+e2, y+e2, z+e2);
	glVertex3f(x+e2, y-e2, z+e2);
	glVertex3f(x+e2, y-e2, z-e2);
	glEnd();
}

void scene(void)
{
	glColor3f (1.0, 1.0, 1.0);
	glPushMatrix();
	glRotatef((timer*2)%180, 0., 1., 0);
	cube(0, 10*cos(radians(timer % 360)), 0, 2);
	glPopMatrix();

	glColor3f (1., 0., 0.);
	cube(10, 0, 0, 2+sin(radians(timer % 360)));

	glColor3f(0., 0., 1.);
	cube(0, 0, 10, 2+cos(radians(timer % 180)));
}

void display(void)
{
    glClear(GL_COLOR_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);		// selectionne la matrice de la scene
	glLoadIdentity();				// reinitialise les transformations
	glRotatef(-camera_angle, 0., 1., 0);
	glTranslatef(-camera_x, -camera_y, -camera_z);

	scene();

    glFlush();
    glutSwapBuffers();
}

void anime(void)
{
	int c= clock();
	if((c - timer_start)*timer_rate > CLOCKS_PER_SEC)
	{
		timer++;
		timer_start= c;
		glutPostRedisplay();
	}
}

void keyboard(unsigned char key, int x, int y)
{
	switch(key)
	{
		case GLUT_KEY_UP:
			camera_z-= cos(radians(camera_angle));
			camera_x-= sin(radians(camera_angle));
		break;

		case GLUT_KEY_DOWN:
			camera_z+= cos(radians(camera_angle));
			camera_x+= sin(radians(camera_angle));
		break;

		case GLUT_KEY_LEFT:
			// camera_x-= 1.;
			// rotation de la camera sur Y-
			camera_angle+= 1.;
			if(camera_angle >= 360.)
				camera_angle= 0.;
		break;

		case GLUT_KEY_RIGHT:
			// camera_x+= 1.;
			// rotation de la camera sur Y+
			camera_angle-= 1.;
			if(camera_angle < 0.)
				camera_angle= 359.;
		break;

		default:
		break;
	}

	glutPostRedisplay();
}


void init (void)
{
    glClearColor (0.0, 0.0, 0.0, 0.0);

    glMatrixMode(GL_PROJECTION);	// selectionne la matrice de visualisation
    glLoadIdentity();				// reintialise les transformations

#if 0
    glOrtho(-1000.0, 1000.0, -1000.0, 1000.0, -1000.0, 1000.0);	// definit une projection orthographique
#else
	gluPerspective(50., 1., 1., 100.);
#endif

	camera_x= 0.;
	camera_y= 0.;
	camera_z= 50.;
	camera_angle= 0.;

	timer= 0;
	timer_start= clock();
	timer_rate= 50;
}

/*
 *  Declare initial window size, position, and display mode
 *  (single buffer and RGBA).  Open window with "hello"
 *  in its title bar.  Call initialization routines.
 *  Register callback function to display graphics.
 *  Enter main loop and process events.
 */
int main(int argc, char** argv)
{
    glutInit(&argc, argv);
    glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB);
    glutInitWindowSize (400, 400);
    glutInitWindowPosition (100, 100);
    glutCreateWindow ("hello");
    init();
    glutSpecialFunc(keyboard);
    glutDisplayFunc(display);
    glutIdleFunc(anime);
    glutMainLoop();
    return 0;
}

